/**
 * @file fileio.h File input/output API
 * 
 * $Id: fileio.h,v 1.3 2001/06/19 09:59:50 chipx86 Exp $
 *
 * @Copyright (C) 1999-2001 The GNUpdate Project.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef _LIBCOMPREX_FILEIO_H_
#define _LIBCOMPREX_FILEIO_H_

#ifdef __cplusplus
extern "C" {
#endif

typedef enum   _CxDepth     CxDepth;      /**< Decompression depth. */
typedef struct _CxFile      CxFile;       /**< A single file.       */
typedef struct _CxFileBatch CxFileBatch;  /**< A group of files.    */

#include <libcomprex/module.h>
#include <stdio.h>

/**
 * @enum _CxDepth fileio.h libcomprex/fileio.h
 * 
 * A depth of decompression.
 *
 * @see cxOpen()
 */
enum _CxDepth
{
	CX_DEPTH_NONE,     /**< Don't recurse at all.               */
	CX_DEPTH_SINGLE,   /**< Decompress the top-level file once. */
	CX_DEPTH_ALL       /**< Decompress recursively.             */
};

/**
 * @struct _CxFile fileio.h libcomprex/fileio.h
 *
 * A single file.
 *
 * @see CxFileBatch
 */
struct _CxFile
{
	/**
	 * The name of the file.
	 *
	 * In an example path of /path/to/my-file,
	 * filename would be my-file.
	 */
	char *filename;

	/**
	 * Physical path information.
	 *
	 * This structure contains information regarding the actual files
	 * stored on the harddrive.
	 */
	struct
	{
		/**
		 * The base path where this file is located.
		 *
		 * In an example path of /path/to/my-file,
		 * basePath would be /path/to.
		 */
		char *basePath;

		/**
		 * The full path name where this file is located.
		 *
		 * In an example path of /path/to/my-file,
		 * fullPath would be /path/to/my-file
		 */
		char *fullPath;

	} physical;

	/**
	 * Internal path information.
	 *
	 * This structure contains information regarding the paths of
	 * the files as they are stored within the archive.
	 */
	struct
	{
		/**
		 * The base path where this file is located inside the archive.
		 *
		 * In an example path of /path/to/my-file,
		 * basePath would be /path/to.
		 */
		char *basePath;

		/**
		 * The full path name where this file is located inside the archive.
		 *
		 * In an example path of /path/to/my-file,
		 * fullPath would be /path/to/my-file
		 */
		char *fullPath;

	} internal;

	/**
	 * The file pointer.
	 *
	 * This should never be manipulated directly, except in modules where
	 * appropriate. Use the libcomprex API for working with file
	 * input/output.
	 */
	FILE *fp;

	CxFileBatch *batch;        /**< The owner file batch.          */
	
	CxFile *prev;              /**< The previous file in the list. */
	CxFile *next;              /**< The next file in the list.     */
};

/**
 * A batch of files.
 *
 * Every CxFile must have one of these attached to it. Even individual
 * files have CxFileBatch's attached.
 *
 * @see CxFile
 */
struct _CxFileBatch
{
	/**
	 * The name of the archive, or if it's an individual file, the name
	 * of the file.
	 *
	 * In an example path of /path/to/my-archive.gz,
	 * filename would be my-archive.gz.
	 */
	char *filename;

	/**
	 * Physical path information.
	 *
	 * This structure contains information regarding the actual files
	 * stored on the harddrive.
	 */
	struct
	{
		/**
		 * The base path where this archive or file is located.
		 *
		 * In an example path of /path/to/my-archive.gz,
		 * basePath would be /path/to.
		 */
		char *basePath;

		/**
		 * The full path name where this archive or file is located.
		 *
		 * In an example path of /path/to/my-archive.gz,
		 * fullPath would be /path/to/my-archive.gz.
		 */
		char *fullPath;

	} physical;

	/**
	 * The number of files in this batch.
	 */
	unsigned int fileCount;

	/**
	 * Specifies whether or not this archive should be deleted when
	 * closed.
	 * 
	 * If this is not 0, this file can be removed when the structure
	 * is freed.
	 *
	 * Note: This should never be touched outside of libcomprex!
	 * 
	 * @todo It may be a good idea to find an alternative method
	 *       later, but this works for now.
	 */
	char discardable;
	
	CxFile *firstFile;   /**< The first file in the batch. */
	CxFile *lastFile;    /**< The last file in the batch.  */
};

CxFile *cxOpen(const char *path, const char *mode, CxDepth depth);
void cxClose(CxFile *file);
void cxCloseBatch(CxFileBatch *batch);
void cxCloseAll(CxFile *file);

size_t cxRead(void *ptr, size_t size, size_t nmemb, CxFile *file);
size_t cxWrite(const void *ptr, size_t size, size_t nmemb, CxFile *file);

CxFile *cxPrevFile(CxFile *file);
CxFile *cxNextFile(CxFile *file);
CxFileBatch *cxGetFileBatch(CxFile *file);

void cxClearErr(CxFile *file);

int cxEof(CxFile *file);
int cxError(CxFile *file);
int cxFileNo(CxFile *file);

#ifdef __cplusplus
}
#endif

#endif /* _LIBCOMPREX_FILEIO_H_ */

