/**
 * @file module.h Module API
 * 
 * $Id: module.h,v 1.2 2001/06/04 06:47:32 chipx86 Exp $
 *
 * @Copyright (C) 1999-2001 The GNUpdate Project.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef _LIBCOMPREX_MODULE_H_
#define _LIBCOMPREX_MODULE_H_

#ifdef __cplusplus
extern "C" {
#endif

typedef enum   _CxModuleType CxModuleType;  /**< Module types.          */
typedef struct _CxFileOps    CxFileOps;     /**< File operations.       */
typedef struct _CxSchemeOps  CxSchemeOps;   /**< URI scheme operations. */
typedef struct _CxModule     CxModule;      /**< A loadable module.     */

#include <ltdl.h>
#include <libcomprex/status.h>
#include <libcomprex/fileio.h>

/**
 * A module type.
 *
 * These are the types of modules that libcomprex can dynamically load.
 */
enum _CxModuleType
{
	CX_MODULE_FILE,    /**< File handler module. */
	CX_MODULE_SCHEME   /**< URI scheme module.   */
};

/**
 * @struct _CxFileOps module.h libcomprex/module.h
 *
 * File operations structure.
 *
 * This structure is to be filled out by all modules that handle
 * comression/decompression of files.
 *
 * @see CxModuleType
 * @see CxSchemeOps
 */
struct _CxFileOps
{
	/**
	 * Compresses a file or batch of files.
	 *
	 * @param batch       The file batch to compress.
	 * @param outFilename The new filename of the compressed archive.
	 *
	 * @return The status of the oepration.
	 *
	 * @see CxStatus
	 */
	CxStatus (*compress)(CxFileBatch *batch, const char *outFilename);

	/**
	 * Decompresses an archive and returns a file batch.
	 *
	 * @param batch   The file batch to fill out.
	 * @param outPath The directory to decompress to.
	 * 
	 * @return The status of the operation.
	 *
	 * @see CxStatus
	 */
	CxStatus (*decompress)(CxFileBatch *batch, const char *outPath);
	
	/**
	 * Determines if the module supports the specified file extension.
	 *
	 * This should just give a best guess. All string comparisons should
	 * be case-insensitive.
	 * 
	 * @param ext The extension (without the leading '.')
	 * 
	 * @return 1 if the extension is supported; 0 otherwise.
	 */
	char (*supportsExtension)(const char *ext);
};

/**
 * @struct _CxSchemeOps module.h libcomprex/module.h
 *
 * Scheme operations structure.
 *
 * This structure is to be filled out by all modules that handle
 * the retrieval of files from non-local locations.
 *
 * @see CxModuleType
 * @see CxFileOps
 */
struct _CxSchemeOps
{
	/**
	 * Retrieves a file from a non-local source.
	 *
	 * @param scheme      The scheme part of the URI.
	 * @param path        The path part of the URI.
	 * @param outFilename The destination filename.
	 *
	 * @return The status of the operation.
	 */
	CxStatus (*get)(const char *scheme, const char *path,
					const char *outFilename);
	
	/**
	 * Determines if the module supports the specified scheme.
	 *
	 * @param scheme The URI scheme to check.
	 * 
	 * @return 1 if the scheme is supported; 0 otherwise.
	 */
	char (*supports)(const char *scheme);
};

/**
 * @struct _CxModule module.h libcomprex/module.h
 * 
 * A file or scheme module.
 *
 * @see CxModuleType
 * @see CxFileOps
 * @see CxSchemeOps
 */
struct _CxModule
{
	CxModuleType type;          /**< The type of module.              */

	char *filename;             /**< The filename of the module.      */
	char *name;                 /**< The name of the module.          */
	lt_dlhandle handle;         /**< The libtool handle.              */

	unsigned int refCount;      /**< The module's reference count.    */

	union
	{
		CxFileOps   *file;      /**< File operations.                 */
		CxSchemeOps *scheme;    /**< Scheme operations.               */

	} ops;                      /**< Operations.                      */

	CxModule *prev;             /**< The previous module in the list. */
	CxModule *next;             /**< The next module in the list.     */
};

/**
 * Loads the module of the specified name and type.
 *
 * @param name The name of the module (without ".so")
 * @param type The type of module.
 * 
 * @return A CxModule structure representing the module.
 */
CxModule *cxLoadModule(const char *name, CxModuleType type);

/**
 * Unloads the specified module.
 *
 * @param module The module to unload.
 */
void cxUnloadModule(CxModule *module);

/**
 * Returns the specified module.
 *
 * The module
 *
 * @param name The name of the module.
 * @param type The type of the module.
 *
 * @return A CxModule structure representing the module.
 */
CxModule *cxGetModule(const char *name, CxModuleType type);

/**
 * Returns the first module in the list.
 *
 * This will probably only be useful inside libcomprex.
 *
 * @param type The type of module.
 * 
 * @return The first module in the list.
 */
CxModule *cxGetFirstModule(CxModuleType type);

/**
 * Returns the next module in the list.
 *
 * This will probably only be good inside libcomprex.
 *
 * @param module The current module.
 *
 * @return The next module in the list.
 */
CxModule *cxGetNextModule(CxModule *module);

/**
 * Unloads all modules.
 *
 * This should only be called when you are absolutely done with
 * this library.
 */
void cxCleanupModules();

#ifdef __cplusplus
}
#endif

#endif /* _LIBCOMPREX_MODULE_H_ */

